/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.base.storage.discrete;

import com.google.common.base.Preconditions;
import org.apiguardian.api.API;
import org.apiguardian.api.API.Status;
import grondag.fluidity.api.article.Article;
import grondag.fluidity.api.article.StoredArticleView;
import grondag.fluidity.api.storage.ArticleFunction;
import grondag.fluidity.api.storage.InventoryStore;
import grondag.fluidity.api.storage.StorageListener;
import grondag.fluidity.base.article.StoredDiscreteArticle;
import grondag.fluidity.base.storage.AbstractLazyRollbackStore;
import grondag.fluidity.base.storage.discrete.FixedDiscreteStore.FixedDiscreteArticleFunction;
import grondag.fluidity.base.storage.discrete.helper.DiscreteNotifier;
import grondag.fluidity.impl.article.ArticleImpl;
import grondag.fluidity.impl.article.StackHelper;
import net.minecraft.class_1799;
import net.minecraft.class_2487;

@API(status = Status.EXPERIMENTAL)
public class SingleStackInventoryStore extends AbstractLazyRollbackStore<StoredDiscreteArticle,  SingleStackInventoryStore> implements DiscreteStore, InventoryStore {
	protected class_1799 stack = class_1799.field_8037;
	protected class_1799 cleanStack = class_1799.field_8037;
	protected final StoredDiscreteArticle view = new StoredDiscreteArticle();
	protected final DiscreteNotifier notifier = new DiscreteNotifier(this);

	@Override
	public ArticleFunction getConsumer() {
		return consumer;
	}

	@Override
	public boolean hasConsumer() {
		return true;
	}

	@Override
	public ArticleFunction getSupplier() {
		return supplier;
	}

	@Override
	public boolean hasSupplier() {
		return true;
	}

	protected final FixedDiscreteArticleFunction consumer = createConsumer();

	protected FixedDiscreteArticleFunction createConsumer() {
		return new Consumer();
	}

	protected class Consumer implements FixedDiscreteArticleFunction {
		@Override
		public long apply(Article article, long count, boolean simulate) {

			if(article.isNothing()) {
				return 0;
			}

			final int maxCount = article.toItem().method_7882();

			if(stack.method_7960()) {
				final int n = (int) Math.min(count, maxCount);

				if(!simulate) {
					rollbackHandler.prepareIfNeeded();
					stack = article.toStack(n);
					cleanStack = stack.method_7972();

					if(!listeners.isEmpty()) {
						notifier.notifyAccept(article, 0, n, n);

						if(maxCount != 64) {
							notifier.notifyCapacityChange(maxCount - 64);
						}
					}
				}

				return n;
			} else if(article.matches(stack)) {
				final int n = (int) Math.min(count, article.toItem().method_7882() - stack.method_7947());

				if(!simulate) {
					rollbackHandler.prepareIfNeeded();
					stack.method_7933(n);
					cleanStack = stack.method_7972();

					if(!listeners.isEmpty()) {
						notifier.notifyAccept(article, 0, n, stack.method_7947());
					}
				}

				return n;
			} else {
				return 0;
			}
		}

		@Override
		public TransactionDelegate getTransactionDelegate() {
			return SingleStackInventoryStore.this;
		}

		@Override
		public long apply(int handle, Article item, long count, boolean simulate) {
			// implement in subtypes
			throw new UnsupportedOperationException();
		}
	}

	protected final FixedDiscreteArticleFunction supplier = createSupplier();

	protected FixedDiscreteArticleFunction createSupplier() {
		return new Supplier();
	}

	protected class Supplier implements FixedDiscreteArticleFunction {
		@Override
		public long apply(Article article, long count, boolean simulate) {
			if(article.isNothing() || stack.method_7960() || !article.matches(stack)) {
				return 0;
			}

			final int n = (int) Math.min(count, stack.method_7947());

			if(!simulate) {
				final int oldMax = stack.method_7914();

				rollbackHandler.prepareIfNeeded();
				stack.method_7934(n);
				cleanStack = stack.method_7972();

				if(!listeners.isEmpty()) {
					notifier.notifySupply(article, 0, n, stack.method_7947());

					if(stack.method_7960() && oldMax != 64) {
						notifier.notifyCapacityChange(64 - oldMax);
					}
				}
			}

			return n;
		}

		@Override
		public TransactionDelegate getTransactionDelegate() {
			return SingleStackInventoryStore.this;
		}

		@Override
		public long apply(int handle, Article item, long count, boolean simulate) {
			// implement in subtypes
			throw new UnsupportedOperationException();
		}
	}

	@Override
	public int handleCount() {
		return 1;
	}

	@Override
	public StoredArticleView view(int handle) {
		return view.prepare(handle == 0 ? stack : class_1799.field_8037, handle);
	}

	@Override
	public int method_5439() {
		return 1;
	}

	@Override
	public boolean method_5442() {
		return stack.method_7960();
	}

	@Override
	public boolean isFull() {
		return !stack.method_7960() && stack.method_7947() >= stack.method_7914();
	}

	@Override
	public class_1799 method_5438(int slot) {
		return slot == 0 ? stack : class_1799.field_8037;
	}

	@Override
	public class_1799 method_5434(int slot, int count) {
		if (slot != 0 || stack.method_7960() || count == 0) {
			return class_1799.field_8037;
		}

		rollbackHandler.prepareIfNeeded();
		final int n = Math.min(count, stack.method_7947());

		if(!listeners.isEmpty()) {
			notifier.notifySupply(ArticleImpl.of(stack), 0, n, stack.method_7947() - n);
		}

		final class_1799 result = stack.method_7972();
		result.method_7939(n);
		stack.method_7934(n);
		cleanStack = stack.method_7972();

		return result;
	}

	@Override
	public class_1799 method_5441(int slot) {
		if (slot != 0 || stack.method_7960()) {
			return class_1799.field_8037;
		}

		rollbackHandler.prepareIfNeeded();

		if(!listeners.isEmpty()) {
			notifier.notifySupply(ArticleImpl.of(stack), 0, stack.method_7947(), 0);
		}

		final class_1799 result = stack;
		stack = class_1799.field_8037;
		cleanStack = class_1799.field_8037;

		return result;
	}

	@Override
	public void method_5447(int slot, class_1799 newStack) {
		Preconditions.checkElementIndex(slot, 1, "Invalid slot number");

		if (StackHelper.areItemsEqual(newStack, stack)) {
			if(newStack.method_7947() == stack.method_7947()) {
				return;
			} else {
				final int delta = newStack.method_7947() - stack.method_7947();

				if(!listeners.isEmpty()) {
					if(delta > 0) {
						notifier.notifyAccept(ArticleImpl.of(stack), 0, delta, newStack.method_7947());
					} else {
						notifier.notifySupply(ArticleImpl.of(stack), 0, -delta, newStack.method_7947());
					}
				}
			}
		} else {
			if(!listeners.isEmpty()) {
				notifier.notifySupply(ArticleImpl.of(stack), 0, stack.method_7947(), 0);
				notifier.notifyAccept(ArticleImpl.of(newStack), 0, newStack.method_7947(), newStack.method_7947());
			}
		}

		rollbackHandler.prepareIfNeeded();
		stack = newStack;
		cleanStack = stack.method_7972();
	}

	@Override
	public void method_5448() {
		if (!stack.method_7960()) {
			rollbackHandler.prepareIfNeeded();

			if(!listeners.isEmpty()) {
				notifier.notifySupply(ArticleImpl.of(stack), 0, stack.method_7947(), 0);
			}

			stack = class_1799.field_8037;
			cleanStack = class_1799.field_8037;
		}
	}

	@Override
	protected Object createRollbackState() {
		return stack.method_7972();
	}

	@Override
	protected void applyRollbackState(Object state, boolean isCommitted) {
		if(!isCommitted) {
			stack = (class_1799) state;
			cleanStack = stack.method_7972();
		}
	}

	@Override
	protected void sendFirstListenerUpdate(StorageListener listener) {
		listener.onCapacityChange(this, stack.method_7914());
		listener.onAccept(this, 0, ArticleImpl.of(stack), stack.method_7947(), stack.method_7947());
	}

	@Override
	protected void sendLastListenerUpdate(StorageListener listener) {
		listener.onSupply(this, 0, ArticleImpl.of(stack), stack.method_7947(), 0);
		listener.onCapacityChange(this, 0);
	}

	@Override
	public long count() {
		return stack.method_7947();
	}

	@Override
	public long capacity() {
		return stack.method_7960() ? 64 : stack.method_7914();
	}

	@Override
	public class_2487 writeTag() {
		return stack.method_7969();
	}

	@Override
	public void readTag(class_2487 tag) {
		stack = class_1799.method_7915(tag);
		cleanStack = stack.method_7972();
	}

	@Override
	protected void onListenersEmpty() {
		// NOOP
	}

	@Override
	public void method_5431() {
		if (StackHelper.areItemsEqual(stack, cleanStack)) {
			if(stack.method_7947() == cleanStack.method_7947()) {
				return;
			} else {
				final int delta = stack.method_7947() - cleanStack.method_7947();

				if(!listeners.isEmpty()) {
					if(delta > 0) {
						notifier.notifyAccept(ArticleImpl.of(stack), 0, delta, stack.method_7947());
					} else {
						notifier.notifySupply(ArticleImpl.of(stack), 0, -delta, stack.method_7947());
					}
				}
			}
		} else if  (!listeners.isEmpty()) {
			notifier.notifySupply(ArticleImpl.of(cleanStack), 0, cleanStack.method_7947(), 0);
			notifier.notifyAccept(ArticleImpl.of(stack), 0, stack.method_7947(), stack.method_7947());
		}

		cleanStack = stack.method_7972();
	}
}
