/*******************************************************************************
 * Copyright 2019, 2020 grondag
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 ******************************************************************************/
package grondag.fluidity.impl.article;

import java.util.function.BiConsumer;
import java.util.function.Function;
import net.minecraft.class_1792;
import net.minecraft.class_2378;
import net.minecraft.class_2481;
import net.minecraft.class_2519;
import net.minecraft.class_2520;
import net.minecraft.class_2540;
import net.minecraft.class_2960;
import net.minecraft.class_3611;
import com.google.common.base.Preconditions;
import org.apiguardian.api.API;
import org.apiguardian.api.API.Status;
import grondag.fluidity.api.article.ArticleType;
import grondag.fluidity.api.article.ArticleTypeRegistry;

@API(status = Status.INTERNAL)
public class ArticleTypeImpl<T> implements ArticleType<T> {
	final Class<T> clazz;
	final boolean isBulk;
	final boolean isFluid;
	final boolean isItem;
	final BiConsumer<T, class_2540> packetWriter;
	final Function<class_2540, T> packetReader;
	final Function<class_2520, T> tagReader;
	final Function<T, class_2520> tagWriter;
	final Function<T, String> keyFunction;

	ArticleTypeImpl(BuilderImpl<T> builder) {
		this.clazz = builder.clazz;
		this.isBulk = builder.isBulk;
		this.packetWriter = builder.packetWriter;
		this.packetReader = builder.packetReader;
		this.tagReader = builder.tagReader;
		this.tagWriter = builder.tagWriter;
		this.keyFunction = builder.keyFunction;
		this.isFluid = clazz == class_3611.class;
		this.isItem = clazz == class_1792.class;
	}

	@Override
	public T cast(Object from) {
		return clazz.cast(from);
	}

	@Override
	public boolean isFluid() {
		return isFluid;
	}

	@Override
	public boolean isItem() {
		return isItem;
	}

	@Override
	public boolean isBulk() {
		return isBulk;
	}

	@Override
	public class_2520 toTag() {
		return class_2519.method_23256(ArticleTypeRegistry.instance().getId(this).toString());
	}

	@Override
	public void toPacket(class_2540 buf) {
		buf.method_10804(ArticleTypeRegistryImpl.INSTANCE.getRawId(this));
	}

	private static class BuilderImpl<U> implements Builder<U> {
		private final Class<U> clazz;
		private boolean isBulk = false;
		private BiConsumer<U, class_2540> packetWriter;
		private Function<class_2540, U> packetReader;
		private Function<U, class_2520> tagWriter;
		private Function<class_2520, U> tagReader;
		private Function<U, String> keyFunction;

		BuilderImpl(Class<U> clazz) {
			this.clazz = clazz;
		}

		@Override
		public Builder<U> bulk(boolean isBulk) {
			this.isBulk = isBulk;
			return this;
		}

		@Override
		public ArticleType<U> build() {
			Preconditions.checkNotNull(packetWriter, "Packet writer required to build article types.");
			Preconditions.checkNotNull(packetReader, "Packet reader required to build article types.");
			Preconditions.checkNotNull(tagWriter, "Tag writer required to build article types.");
			Preconditions.checkNotNull(tagReader, "Tag reader required to build article types.");
			return new ArticleTypeImpl<>(this);
		}

		@Override
		public Builder<U> resourceTagWriter(Function<U, class_2520> tagWriter) {
			this.tagWriter = tagWriter;
			return this;
		}

		@Override
		public Builder<U> resourceTagReader(Function<class_2520, U> tagReader) {
			this.tagReader = tagReader;
			return this;
		}

		@Override
		public Builder<U> resourcePacketWriter(BiConsumer<U, class_2540> packetWriter) {
			this.packetWriter = packetWriter;
			return this;
		}

		@Override
		public Builder<U> resourcePacketReader(Function<class_2540, U> packetReader) {
			this.packetReader = packetReader;
			return this;
		}

		@Override
		public Builder<U> translationKeyFunction(Function<U, String> keyFunction) {
			this.keyFunction = keyFunction;
			return this;
		}
	}

	public static final ArticleType<class_1792> ITEM = ArticleTypeRegistryImpl.INSTANCE.add("fluidity:item",
			builder(class_1792.class)
			.bulk(false)
			.resourceTagWriter(r -> class_2519.method_23256(class_2378.field_11142.method_10221(r).toString()))
			.resourceTagReader(t -> class_2378.field_11142.method_10223(new class_2960(t.method_10714())))
			.resourcePacketWriter((r, p) -> p.method_10804(class_2378.field_11142.method_10249(r)))
			.resourcePacketReader(p -> class_2378.field_11142.method_10200(p.method_10816()))
			.translationKeyFunction(i -> i.method_7876())
			.build());

	public static final ArticleType<class_3611> FLUID = ArticleTypeRegistryImpl.INSTANCE.add("fluidity:fluid", builder(class_3611.class)
			.bulk(true)
			.resourceTagWriter(r -> class_2519.method_23256(class_2378.field_11154.method_10221(r).toString()))
			.resourceTagReader(t -> class_2378.field_11154.method_10223(new class_2960(t.method_10714())))
			.resourcePacketWriter((r, p) -> p.method_10804(class_2378.field_11154.method_10249(r)))
			.resourcePacketReader(p -> class_2378.field_11154.method_10200(p.method_10816()))
			.translationKeyFunction(f -> f.method_15785().method_15759().method_26204().method_9539())
			.build());

	public static final ArticleType<Void> NOTHING = ArticleTypeRegistryImpl.INSTANCE.add("fluidity:nothing", builder(Void.class)
			.bulk(false)
			.resourceTagWriter(r -> class_2481.field_21026)
			.resourceTagReader(t -> null)
			.resourcePacketWriter((r, p) -> {})
			.resourcePacketReader(p -> null)
			.translationKeyFunction(n -> "fluidity:nothing")
			.build());

	public static <V> BuilderImpl<V> builder(Class<V> clazz) {
		return new BuilderImpl<>(clazz);
	}

	@SuppressWarnings("unchecked")
	public static <T> ArticleTypeImpl<T> fromTag(class_2520 tag) {
		return ArticleTypeRegistryImpl.INSTANCE.get(tag.method_10714());
	}

	@SuppressWarnings("unchecked")
	public static <T> ArticleTypeImpl<T> fromPacket(class_2540 buf) {
		return ArticleTypeRegistryImpl.INSTANCE.get(buf.method_10816());
	}

	public static void init() {
		// force loading
	}
}
